// CASTORFile.cpp: implementation of the CASTORFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "CASTORFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

	/** Constructor
	 *
	 * @param inputFile The file to read input from
	 * @param outputFile The file to write output to
	 *
	 */
CASTORFile::CASTORFile(std::string inputFile, std::string outputFile)
{
	m_InFile.open(inputFile.c_str(), std::ios::in);
	setInFileSize();

	m_OutFile.open(outputFile.c_str(), std::ios::out);
	m_pCharVeh = m_VehCharArry;
}

	/**Default Destructor*/
CASTORFile::~CASTORFile()
{
	m_InFile.close();
	m_OutFile.flush();

	m_Vehicles.clear();
}

	/** Reads a line representing a truck from a CASTOR file
	 *
	 * @return A newly created truck
	 * 
	 * This function reads the next line available from a CASTOR format file
	 * and stores the information in a string. If the string is not null, a
	 * truck is created with the properties specified by the line from the
	 * file, and the truck is returned
	 */
Vehicle* CASTORFile::readLine()
{
	std::string str;
	std::getline(m_InFile, str, '\n');

	if(str != "")
	{
		m_CurLine++;
		Vehicle* pVeh;
		int GVW = atoi( str.substr(21,4).c_str() ); // TRUCK_WEIGHT_THRESHOLD in kg/100

		if(GVW >= TRUCK_WEIGHT_THRESHOLD * KG100_TO_KN)
			pVeh = new Truck();
		else
			pVeh = new Car();

		pVeh->createCASTORVehicle(str);
		return pVeh;
	}

	return NULL;
}

	/** Writes a line representing a truck to a CASTOR file
	 *
	 * @param pVeh The vehicle to write to file
	 *
	 * This function takes a vehicle as a parameter, and calls this vehicle's
	 * doCASTORDATA function, in order to obtain a string which represents all
	 * of the vehicle's physical properties. It then writes this string to a 
	 * CASTOR format file
	 */
void CASTORFile::writeLine(Vehicle *pVeh)
{
	pVeh->writeCASTORData(m_pCharVeh);
	m_OutFile << m_pCharVeh << '\n';
}
