// ConfigData.cpp: implementation of the CConfigData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "ConfigData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CConfigData::CConfigData()
{
	m_CommentString = "//";
	SetDefaults();
}

CConfigData::~CConfigData()
{

}

// the global config data object

CConfigData g_ConfigData;

bool CConfigData::ReadData(std::string inFile)
{
	if( m_CSV.OpenFile(inFile, ",") )
	{
		ExtractData();
		return true;
	}
	return false;
}

void CConfigData::ExtractData()
{
		string str;

		str = GetNextDataLine();	IDM.LANECHANGE_MIN_GAP			= stringToDouble(str);
		str = GetNextDataLine();	IDM.MIN_SPACE_FOR_NEXT_VEHICLE	= stringToDouble(str);
		str = GetNextDataLine();	IDM.SAFE_BRAKING				= stringToDouble(str);
		str = GetNextDataLine();	IDM.T_DELAY						= stringToDouble(str);

		str = GetNextDataLine();	Road.ROAD_END_BUFFER								= stringToInt(str);
		str = GetNextDataLine();	Road.RoadFeatures.SpeedLimit.SPEEDLIMIT_MIN			= stringToInt(str);
		str = GetNextDataLine();	Road.RoadFeatures.SpeedLimit.SPEEDLIMIT_MAX			= stringToInt(str);
		str = GetNextDataLine();	Road.RoadFeatures.Gradient.MAX_SLOPE				= stringToInt(str);
		str = GetNextDataLine();	Road.RoadFeatures.Gradient.IDM_Modifiers.A_FLAG		= stringToBool(str);
		str = GetNextDataLine();	Road.RoadFeatures.Gradient.IDM_Modifiers.A_MIN		= stringToDouble(str);
		str = GetNextDataLine();	Road.RoadFeatures.Gradient.IDM_Modifiers.B_FLAG		= stringToBool(str);
		str = GetNextDataLine();	Road.RoadFeatures.Gradient.IDM_Modifiers.T_SLOPE	= stringToDouble(str);
		str = GetNextDataLine();	Road.RoadFeatures.Gradient.IDM_Modifiers.V0_MIN		= stringToDouble(str);
		str = GetNextDataLine();	Road.RoadFeatures.Gradient.IDM_Modifiers.V0_SLOPE	= stringToDouble(str);

		str = GetNextDataLine();	Time.DAYS_PER_MT	= stringToInt(str);
		str = GetNextDataLine();	Time.MTS_PER_YR		= stringToInt(str);
		
		str = GetNextDataLine();	VehicleID.CRANE_AVERAGE_SPACING		= stringToDouble(str);
		str = GetNextDataLine();	VehicleID.CRANE_MAX_SPACING			= stringToDouble(str);
		str = GetNextDataLine();	VehicleID.LOWLOADER_MIN_MAX_SPACING	= stringToDouble(str);
		str = GetNextDataLine();	VehicleID.SMALL_TRUCK_NO_AXLES		= stringToInt(str);
		str = GetNextDataLine();	VehicleID.TRUCK_WEIGHT_THRESHOLD	= stringToInt(str);

		str = GetNextDataLine();	View.MIN_VIEW_SCALE					= stringToDouble(str);	
		str = GetNextDataLine();	View.DATATIP_DELAY					= stringToDouble(str);
		str = GetNextDataLine();	View.Colours.Vehicles.VEH_COLOR_CAR				= stringToColour(str);
		str = GetNextDataLine();	View.Colours.Vehicles.VEH_COLOR_SMALLTRUCK		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.Vehicles.VEH_COLOR_LARGETRUCK		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.Vehicles.VEH_COLOR_CRANE			= stringToColour(str);
		str = GetNextDataLine();	View.Colours.Vehicles.VEH_COLOR_LOWLOADER		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_BACKGRND			= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_ROAD_SURFACE		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_ROAD_LINES		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_RULER_LINES		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_RULER_TEXT		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_LEGEND_TEXT		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_DET_OUTPUT		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_DET_FLOWDENSITY	= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_DET_HEADWAY		= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_DET_COMPOSITION	= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_FEAT_SPEEDLIMIT	= stringToColour(str);
		str = GetNextDataLine();	View.Colours.DrawElements.CLR_FEAT_GRADIENT		= stringToColour(str);
		
		m_CSV.CloseFile();
}

string CConfigData::GetNextDataLine()
{
	string line;
	m_CSV.getline(line);
	while(line.substr(0,2) == m_CommentString)
		m_CSV.getline(line);

	return line;
}



void CConfigData::SetDefaults()
{
	IDM.LANECHANGE_MIN_GAP			= 2.0;
	IDM.MIN_SPACE_FOR_NEXT_VEHICLE	= 27.0;
	IDM.SAFE_BRAKING				= -12.0;
	IDM.T_DELAY						= 1.6;


	Road.ROAD_END_BUFFER								= 1000;
	
	Road.RoadFeatures.MAX_NO_ROAD_FEATURES				= 10;
	
	Road.RoadFeatures.SpeedLimit.SPEEDLIMIT_MIN			= 10;
	Road.RoadFeatures.SpeedLimit.SPEEDLIMIT_MAX			= 200;
	
	Road.RoadFeatures.Gradient.MAX_SLOPE				= 15;
	Road.RoadFeatures.Gradient.IDM_Modifiers.A_FLAG		= true;
	Road.RoadFeatures.Gradient.IDM_Modifiers.A_MIN		= 0.1;
	Road.RoadFeatures.Gradient.IDM_Modifiers.B_FLAG		= true;
	Road.RoadFeatures.Gradient.IDM_Modifiers.T_SLOPE	= 10.0;
	Road.RoadFeatures.Gradient.IDM_Modifiers.V0_MIN		= 0.1;
	Road.RoadFeatures.Gradient.IDM_Modifiers.V0_SLOPE	= 0.2;
	

	Time.DAYS_PER_MT	= 25;
	Time.MTS_PER_YR		= 10;
	

	VehicleID.CRANE_AVERAGE_SPACING		= 2.5;
	VehicleID.CRANE_MAX_SPACING			= 4.5;
	VehicleID.LOWLOADER_MIN_MAX_SPACING	= 7.5;
	VehicleID.SMALL_TRUCK_NO_AXLES		= 3;
	VehicleID.TRUCK_WEIGHT_THRESHOLD	= 35;

	
	View.MIN_VIEW_SCALE					= 0.1;
	View.DATATIP_DELAY					= 20.0;
	
	View.Colours.Vehicles.VEH_COLOR_CAR			= RGB(0,0,0);
	View.Colours.Vehicles.VEH_COLOR_SMALLTRUCK	= RGB(0,0,255);
	View.Colours.Vehicles.VEH_COLOR_LARGETRUCK	= RGB(255,0,0);
	View.Colours.Vehicles.VEH_COLOR_CRANE		= RGB(0,255,0);
	View.Colours.Vehicles.VEH_COLOR_LOWLOADER	= RGB(100,100,100);
	
	View.Colours.DrawElements.CLR_BACKGRND			= RGB(255,255,255);
	View.Colours.DrawElements.CLR_ROAD_SURFACE		= RGB(200,200,200);
	View.Colours.DrawElements.CLR_ROAD_LINES		= RGB(0,0,0);
	View.Colours.DrawElements.CLR_RULER_LINES		= RGB(0,0,0);
	View.Colours.DrawElements.CLR_RULER_TEXT		= RGB(0,0,0);
	View.Colours.DrawElements.CLR_LEGEND_TEXT		= RGB(0,0,0);
	View.Colours.DrawElements.CLR_DET_OUTPUT		= RGB(255,255,0);
	View.Colours.DrawElements.CLR_DET_FLOWDENSITY	= RGB(0,255,0);
	View.Colours.DrawElements.CLR_DET_HEADWAY		= RGB(0,0,255);
	View.Colours.DrawElements.CLR_DET_COMPOSITION	= RGB(0,0,0);
	View.Colours.DrawElements.CLR_FEAT_SPEEDLIMIT	= RGB(255,0,0);
	View.Colours.DrawElements.CLR_FEAT_GRADIENT		= RGB(0,255,0);
}

double CConfigData::stringToDouble(string line)
{
	double val;	char data[15]; 
	line.copy(data, 14, 0);  
	val = atof(data);
	return val;
}

int CConfigData::stringToInt(string line)
{
	int val = 0;	char data[15]; 
	line.copy(data, 14, 0);  
	val = atoi(data);
	return val;
}

bool CConfigData::stringToBool(string line)
{
	int val = stringToInt(line);
	if(val == 1)
		return true;
	else
		return false;
}


COLORREF CConfigData::stringToColour(string line)
{
	int red, green, blue;
	string str;
	
	str = m_CSV.getfield(0);	red = stringToInt(str);
	str = m_CSV.getfield(1);	green = stringToInt(str);
	str = m_CSV.getfield(2);	blue = stringToInt(str);

	return RGB(red,green,blue);
}


