// ConfigData.h: interface for the CConfigData class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CONFIGDATA_H__F17D3AEB_F9C5_4B29_A869_E50D84EA9B0A__INCLUDED_)
#define AFX_CONFIGDATA_H__F17D3AEB_F9C5_4B29_A869_E50D84EA9B0A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "stdafx.h"
#include "CSVParse.h"

	/**A class for containing all of the configuration prameters for the program*/

class CConfigData  
{
public:
	void SetDefaults();
	CConfigData();
	virtual ~CConfigData();
	
	bool ReadData(std::string inFile);

	// General IDM Parameters
	struct IDM_Config
	{
		double LANECHANGE_MIN_GAP;
		double MIN_SPACE_FOR_NEXT_VEHICLE;
		double SAFE_BRAKING;
		double T_DELAY;
	} IDM;

	// Road stuff
	struct Road_Config
	{
		int ROAD_END_BUFFER;
		
		struct RoadFeatures_Config
		{
			int MAX_NO_ROAD_FEATURES;

			// Speed limit info
			struct SpeedLimit_Config
			{
				int	SPEEDLIMIT_MIN;
				int	SPEEDLIMIT_MAX;
			} SpeedLimit;
			
			// Constants for how Gradient affects vehicles
			struct Gradient_Config
			{
				int		MAX_SLOPE;
				struct IDM_Param_Modifiers
				{
					double	T_SLOPE;
					double	V0_SLOPE;
					double	V0_MIN;
					double	A_MIN;
					bool	A_FLAG;
					bool	B_FLAG;
				} IDM_Modifiers;
			} Gradient;
		} RoadFeatures;
	} Road;

	// Time output constants
	struct Time_Config
	{
		int MTS_PER_YR;
		int DAYS_PER_MT;
	} Time;

	// Truck identification constants
	struct VehicleID_Config
	{
		int		TRUCK_WEIGHT_THRESHOLD;
		int		SMALL_TRUCK_NO_AXLES;
		int		CRANE_MAX_SPACING;
		double	CRANE_AVERAGE_SPACING;
		double	LOWLOADER_MIN_MAX_SPACING;
	} VehicleID;

	// Screen parameters
	struct View_Config
	{
		// Viewing 
		double MIN_VIEW_SCALE;
		double DATATIP_DELAY;
		// Colours for vehicles
		struct View_Colours
		{
			struct Col_Vehicles
			{
				COLORREF VEH_COLOR_CAR;
				COLORREF VEH_COLOR_SMALLTRUCK;
				COLORREF VEH_COLOR_LARGETRUCK;
				COLORREF VEH_COLOR_CRANE;
				COLORREF VEH_COLOR_LOWLOADER;
			} Vehicles;

			// Colours for elements
			struct Col_DrawElements
			{
				COLORREF CLR_BACKGRND;
				COLORREF CLR_ROAD_SURFACE;
				COLORREF CLR_ROAD_LINES;
				COLORREF CLR_RULER_LINES;
				COLORREF CLR_RULER_TEXT;
				COLORREF CLR_LEGEND_TEXT;
				COLORREF CLR_DET_OUTPUT;
				COLORREF CLR_DET_FLOWDENSITY;
				COLORREF CLR_DET_HEADWAY;
				COLORREF CLR_DET_COMPOSITION;
				COLORREF CLR_FEAT_SPEEDLIMIT;
				COLORREF CLR_FEAT_GRADIENT;
			} DrawElements;
		} Colours;
	} View;

private:
	void ExtractData();
	COLORREF stringToColour(string line);
	string m_CommentString;
	string GetNextDataLine();
	CCSVParse m_CSV;
	double stringToDouble(string line);
	int stringToInt(string line);
	bool stringToBool(string line);
};

#endif // !defined(AFX_CONFIGDATA_H__F17D3AEB_F9C5_4B29_A869_E50D84EA9B0A__INCLUDED_)



