// Define constants for EvolveTraffic

#ifndef Constants_h
#define Constants_h

#include "stdafx.h"	// this is for the defs of WORD etc

	/***************************************************************
	 *FUNDAMENTAL CONSTANTS - DO NOT VARY BETWEEN PROGRAM INSTANCES*
	 ***************************************************************/

	// Supported Distributions
	const WORD DIST_EXPONENTIAL		= 200U;
	const WORD DIST_LOGNORMAL		= 201U;
	const WORD DIST_GAMMA			= 202U;
	const WORD DIST_GUMBEL			= 203U;
	const WORD DIST_POISSON			= 204U;
	const WORD DIST_GEV				= 205U;
	const WORD DIST_NORMAL			= 206U;
	const WORD DIST_CONST			= 207U;

	// IDM Parameter IDs
	const WORD IDM_PARAM_T			= 300U;
	const WORD IDM_PARAM_A			= 301U;
	const WORD IDM_PARAM_B			= 302U;
	const WORD IDM_PARAM_S0			= 303U;
	const WORD IDM_PARAM_S1			= 304U;
	const WORD IDM_PARAM_V0			= 305U;
	const WORD IDM_PARAM_DELTA		= 306U;
	const WORD IDM_PARAM_POLITE		= 307U;
	const WORD IDM_PARAM_BIAS		= 308U;
	const WORD IDM_PARAM_DELTAATH	= 309U;

	// Vehicle types
	const WORD VEH_ID_CAR			= 0;
	const WORD VEH_ID_SMALLTRUCK	= 1;
	const WORD VEH_ID_LARGETRUCK	= 2;
	const WORD VEH_ID_CRANE			= 3;
	const WORD VEH_ID_LOWLOADER		= 4;

	// Road Features IDs
	const WORD FEAT_SPEEDLIMIT		= 400U;
	const WORD FEAT_GRADIENT		= 401U;
	
	// Max number of axles in a CASTOR file
	const int CASTOR_MAX_AXLES		= 9;	
	
	// Program Version for Serialization
	const UINT VERSION_NUMBER		= 1;
	
	// Conversion of metres per second to kilometres per hour
	const double M_PER_S_TO_KM_PER_H	= 3.6;
	const double KM_PER_H_TO_M_PER_S	= 0.27777778;

	// Conversion of kg/100 to kN
	const double KG100_TO_KN			= 0.981;

	const int SECS_PER_HOUR				= 3600;
	const int HOURS_PER_DAY				= 24;
	const int MINS_PER_HOUR				= 60;
	const int SECS_PER_MIN				= 60;
	
	// Fixed number of rows/columns
	const int FIXED_ROWS				= 1;
	const int FIXED_COLUMNS				= 1;

	// Stat Detectors
	const WORD METRICS_VEH_ALL			= 500U;
	const WORD METRICS_VEH_CAR			= 501U;
	const WORD METRICS_VEH_SMALLTRUCK	= 502U;
	const WORD METRICS_VEH_LARGETRUCK	= 503U;
	const WORD METRICS_VEH_CRANE		= 504U;
	const WORD METRICS_VEH_LOWLOADER	= 505U;
	
	const WORD METRICS_TYPE_FLOWDENSITY	= 510U;
	const WORD METRICS_TYPE_HEADWAY		= 511U;
	const WORD METRICS_TYPE_COMPOSITION	= 512U;
	const WORD METRICS_TYPE_LANE_CHANGE	= 513U;

	// Input File Types
	const WORD SAFT						= 100U;
	const WORD CASTOR					= 101U;

	// Input file buffer size
	const WORD INPUT_FILE_BUFFER_SIZE	= 10U;

	// The refresh rate for OnDraw
	const WORD DRAW_FRAMES_PER_SEC		= 20U;
	
	/***********************************************************
	 *VARIABLE CONSTANTS - CHANGEABLE BETWEEN PROGRAM INSTANCES*
	 ***********************************************************/
/*	
	// Colours for vehicles (which can change)
	const COLORREF VEH_COLOR_CAR		= RGB(0,0,0);
	const COLORREF VEH_COLOR_SMALLTRUCK	= RGB(0,0,255);
	const COLORREF VEH_COLOR_LARGETRUCK	= RGB(255,0,0);
	const COLORREF VEH_COLOR_CRANE		= RGB(0,255,0);
	const COLORREF VEH_COLOR_LOWLOADER	= RGB(100,100,100);

	// Colours for elements
	const COLORREF CLR_BACKGRND			= RGB(255,255,255);
	const COLORREF CLR_ROAD_SURFACE		= RGB(200,200,200);
	const COLORREF CLR_ROAD_LINES		= RGB(0,0,0);
	const COLORREF CLR_RULER_LINES		= RGB(0,0,0);
	const COLORREF CLR_RULER_TEXT		= RGB(0,0,0);
	const COLORREF CLR_LEGEND_TEXT		= RGB(0,0,0);
	const COLORREF CLR_DET_OUTPUT		= RGB(255,255,0);
	const COLORREF CLR_DET_FLOWDENSITY	= RGB(0,255,0);
	const COLORREF CLR_DET_HEADWAY		= RGB(0,0,255);
	const COLORREF CLR_DET_COMPOSITION	= RGB(0,0,0);
	const COLORREF CLR_FEAT_SPEEDLIMIT	= RGB(255,0,0);
	const COLORREF CLR_FEAT_GRADIENT	= RGB(0,255,0);

	// Road End Buffer
	const int ROAD_END_BUFFER			= 1000;

	// Truck identification constants
	const int TRUCK_WEIGHT_THRESHOLD	= 35;	// in kg/100
	const int SMALL_TRUCK_NO_AXLES		= 3;
	const double CRANE_MAX_SPACING		= 4.5;	// in m
	const double CRANE_AVERAGE_SPACING	= 2.5;	// in m
	const double LOWLOADER_MAX_SPACING	= 7.5;	// in m

	// Viewing param
	const double MIN_VIEW_SCALE			= 0.1;

	// Max no of road features
	const int MAX_NO_FEATURES			= 10;

	// Constants for how Gradient affects vehicles
	const int	 SPEEDLIMIT_MAX			= 200;		// in km/h - the maximum possible speed limit
	const int	 GRADIENT_MAX			= 15;		// In % - the maximum gradient slope allowed
	const double GRADIENT_T_SLOPE		= 10.0;		// %gradient per second increase in T 
	const double GRADIENT_V0_SLOPE		= 0.2;		// %gradient per km/h reduction in V0
	const double GRADIENT_V0_MIN		= 0.1;		// minimum V) in case gradient effect is too large
	const double GRADIENT_A_MIN			= 0.1;		// m/s^2 minimum m_A in case gradient effect is too large
	const bool	 GRADIENT_A_FLAG		= true;		// should we affect the mechanical property A?
	const bool	 GRADIENT_B_FLAG		= true;		// should we affect the mechanical property B?

	// Time output constants
	const WORD MTS_PER_YR				= 10;
	const WORD DAYS_PER_MT				= 25;

	// Some unusual constants to be queried, but as per Treiber's software
	const double SPACE_MIN				= 27.0;
	const double SAFE_BRAKING			= -12.0;
	const double MIN_GAP				= 2.0;
	const double T_DELAY				= 1.6;	// what is this?

*/

#endif //!defined(Constants.h)

// Supported Distributions

//0 - Exponential
//1 - Log-normal
//2 - Gamma
//3 - Gumbel
//4 - Poisson
//5 - GEV
//6 - Normal

// IDM Parameters

// m_T;			// Safe time headway
// m_a;			// Maximum acceleration
// m_b;			// desired decceleration
// m_s0;		// Jam distance s0
// m_s1;		// Jam distance s1
// m_v0;		// desired velocity
// m_delta;		// Acceleration exponent
// polite;		// Politness factor
// biasRight;	// Bias factor for slow lane