// Detector.cpp: implementation of the Detector class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "Detector.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Detector::Detector()
{
	m_TotalTime = 0.0;
	m_CurrentTime = 0.0;
	m_IntervalNo = 0;
	m_MetricsDir = "C:\\EvolveTraffic\\Metrics\\";
}

Detector::~Detector()
{

}
	/** Checks if it is time for the detector to output
	 *
	 * @param step The timestep
	 *
	 * Given the timestep and the time at which the detector should output, this
	 * function is called each iteration. When the output time has been reached,
	 * the detector outputs
	 */
void Detector::CheckForOutputTime(double step)
{
	m_TotalTime += step;
	m_CurrentTime += step;
	if(m_CurrentTime >= m_TimeInterval)
	{
		m_CurrentTime -= m_TimeInterval;	// We reset the timer
		m_IntervalNo++;						// increase the index
		doIntervalOutput();					// we call the virtual write function
	}
}

std::string Detector::MapDetVehTypeToString(WORD DetVehType)
{
	switch(DetVehType)
	{
		case METRICS_VEH_ALL:			return "All";
		case METRICS_VEH_CAR:			return "Car";
		case METRICS_VEH_SMALLTRUCK:	return "ST";
		case METRICS_VEH_LARGETRUCK:	return "LT";
		case METRICS_VEH_CRANE:			return "Crane";
		case METRICS_VEH_LOWLOADER:		return "LL";
		default: return "All"; 
	}
}

std::string Detector::MapDirToString(bool DirPos)
{
	if(m_DirPos)
		return "Pos";
	else
		return "Neg";
}

	/** Finds the arrival time of a vehicle
	 *
	 * @param pVeh The vehicle
	 * @param curTime The current time
	 * @return The time of arrival
	 *
	 * This function finds the time that a vehicle arrived at a detector by
	 * interpolating between simulation steps
	 */
double Detector::FindDetectorArrivalTime(Vehicle *pVeh, const double curTime)
{
	double distBeyondDetector = pVeh->getPos() - m_Location;
	double ArrivalTime = curTime - distBeyondDetector/pVeh->getVelocity();

	return ArrivalTime;
}

int Detector::getLocation()
{
	return m_Location;
}

bool Detector::getDirPos()
{
	return m_DirPos;
}

WORD Detector::getDetectorType()
{
	return m_DetectorType;
}