// EvolveTrafficDoc.cpp : implementation of the CEvolveTrafficDoc class
//

#include "stdafx.h"
#include "EvolveTraffic.h"

#include "EvolveTrafficDoc.h"
#include "RoadFeature.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficDoc

IMPLEMENT_DYNCREATE(CEvolveTrafficDoc, CDocument)

BEGIN_MESSAGE_MAP(CEvolveTrafficDoc, CDocument)
	//{{AFX_MSG_MAP(CEvolveTrafficDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficDoc construction/destruction

CEvolveTrafficDoc::CEvolveTrafficDoc()
{
	m_IDMParams_Car			.setVehicleTypeID(VEH_ID_CAR);
	m_IDMParams_SmallTruck	.setVehicleTypeID(VEH_ID_SMALLTRUCK);
	m_IDMParams_LargeTruck	.setVehicleTypeID(VEH_ID_LARGETRUCK);
	m_IDMParams_Crane		.setVehicleTypeID(VEH_ID_CRANE);
	m_IDMParams_Lowloader	.setVehicleTypeID(VEH_ID_LOWLOADER);
}

CEvolveTrafficDoc::~CEvolveTrafficDoc()
{
	clear();
}

void CEvolveTrafficDoc::clear()
{
	for(int i = 0; i < m_vRoadFeatures.GetSize(); i++)
		delete m_vRoadFeatures.GetAt(i);
	m_vRoadFeatures.RemoveAll();

	for(i = 0; i < m_vStatDetectors.GetSize(); i++)
		delete m_vStatDetectors.GetAt(i);
	m_vStatDetectors.RemoveAll();

}

BOOL CEvolveTrafficDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	m_FileIn = _T("");
	m_FileOut = _T("");
	m_RoadLength = 3000;
	m_NoLanesDirPos = 2;
	m_NoLanesDirNeg = 1;
	m_FileType = CASTOR;
	m_SimTimeStep = 0.1;
	m_DriveOnRight = true;
	m_LocOutputDetectorDirPos = 2500;
	m_LocOutputDetectorDirNeg = 2500;

	CRoadFeature* pFeat = new CRoadFeature;
	m_vRoadFeatures.Add(pFeat);

	CStatDetector* pDet = new CStatDetector;
	m_vStatDetectors.Add(pDet);

	UpdateDerivedMembers(false);	// do not call SetModFlag

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficDoc serialization

void CEvolveTrafficDoc::Serialize(CArchive& ar)
{
	m_vStatDetectors		.Serialize(ar);
	m_vRoadFeatures			.Serialize(ar);

	m_IDMParams_Car			.Serialize(ar);
	m_IDMParams_SmallTruck	.Serialize(ar);
	m_IDMParams_LargeTruck	.Serialize(ar);
	m_IDMParams_Crane		.Serialize(ar);
	m_IDMParams_Lowloader	.Serialize(ar);

	if (ar.IsStoring())
	{
		ar	<< m_FileIn
			<< m_FileOut
			<< m_MetricsDir
			<< m_FileType
			<< m_TrafFileNoLanesDirPos
			<< m_TrafFileNoLanesDirNeg
			<< m_NoDirections
			<< m_NoLanes
			<< m_NoLanesDirPos
			<< m_NoLanesDirNeg
			<< m_RoadLength
			<< m_DriveOnRight
			<< m_SimTimeStep
			<< m_LocOutputDetectorDirPos
			<< m_LocOutputDetectorDirNeg
			<< m_AllowLaneChanging;
	}
	else
	{
		ar	>> m_FileIn
			>> m_FileOut
			>> m_MetricsDir
			>> m_FileType
			>> m_TrafFileNoLanesDirPos
			>> m_TrafFileNoLanesDirNeg
			>> m_NoDirections
			>> m_NoLanes
			>> m_NoLanesDirPos
			>> m_NoLanesDirNeg
			>> m_RoadLength
			>> m_DriveOnRight
			>> m_SimTimeStep
			>> m_LocOutputDetectorDirPos
			>> m_LocOutputDetectorDirNeg
			>> m_AllowLaneChanging;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficDoc diagnostics

#ifdef _DEBUG
void CEvolveTrafficDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CEvolveTrafficDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficDoc commands

void CEvolveTrafficDoc::UpdateDerivedMembers(bool setMod)
{
	m_NoLanes = m_NoLanesDirPos + m_NoLanesDirNeg;
	m_NoDirections = (m_NoLanesDirPos > 0 && m_NoLanesDirNeg > 0) ? 2 : 1;

	if(setMod)
		SetModifiedFlag();	// To implement Save Changes prompts
}

bool CEvolveTrafficDoc::initSim()
{
	CFile FileIn; CFileStatus status;
	bool bFileExists = FileIn.GetStatus((LPCTSTR)m_FileIn,status);
	if(!bFileExists)
	{
		MessageBox(NULL, "The input traffic file does not exist","EvolveTraffic",MB_OK|MB_ICONWARNING);
		return false;
	}
	
	m_Sim.setFileIn( (LPCTSTR)m_FileIn );
	m_Sim.setFileOut( (LPCTSTR)m_FileOut );
	m_Sim.setFileType(m_FileType);
	m_Sim.setNoDirections(m_NoDirections);
	m_Sim.setNoLanes(m_NoLanes);
	m_Sim.setNoLanesDirPos(m_NoLanesDirPos);
	m_Sim.setNoLanesDirNeg(m_NoLanesDirNeg);
	m_Sim.setRoadLength(m_RoadLength);
	m_Sim.setSimTimeStep(m_SimTimeStep);
	
	m_Sim.getRoad()->setIDMParams_Car		(&m_IDMParams_Car);
	m_Sim.getRoad()->setIDMParams_SmallTruck(&m_IDMParams_SmallTruck);
	m_Sim.getRoad()->setIDMParams_LargeTruck(&m_IDMParams_LargeTruck);
	m_Sim.getRoad()->setIDMParams_Crane		(&m_IDMParams_Crane);
	m_Sim.getRoad()->setIDMParams_Lowloader	(&m_IDMParams_Lowloader);

	for(int i = 0; i < m_vRoadFeatures.GetSize(); i++)
		m_Sim.getRoad()->SetSegmentFromFeature( reinterpret_cast<CRoadFeature*>(m_vRoadFeatures.GetAt(i)) );
	
	for(i = 0; i < m_vStatDetectors.GetSize(); i++)
	{
		CStatDetector* pSD = reinterpret_cast<CStatDetector*>(m_vStatDetectors.GetAt(i));
		pSD->setMetricsDir(m_MetricsDir);
		m_Sim.getRoad()->SetMetricDetFromStatDet( pSD, m_NoLanesDirPos, m_NoLanesDirNeg);
	}
	
	m_Sim.getRoad()->setTrafFileNoLanesDirPos(m_TrafFileNoLanesDirPos);
	m_Sim.getRoad()->setTrafFileNoLanesDirNeg(m_TrafFileNoLanesDirNeg);
	m_Sim.getRoad()->setLocOuputDetectorDirPos(m_LocOutputDetectorDirPos);
	m_Sim.getRoad()->setLocOuputDetectorDirNeg(m_LocOutputDetectorDirNeg);
	m_Sim.getRoad()->setAllowLaneChanging(m_AllowLaneChanging);
	m_Sim.getRoad()->setDriveOnRight(m_DriveOnRight);
	
	m_Sim.init();	// must be called last when all other params are set

	return true;
}

// Private Member Sets
void CEvolveTrafficDoc::setRoadLength(int L)
{
	m_RoadLength = L;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setNoLanes(int nl)
{
	m_NoLanes = nl;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setNoDirections(int nd)
{
	m_NoDirections = nd;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setSimTimeStep(double ts)
{
	m_SimTimeStep = ts;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setNoLanesDirPos(int nlpos)
{
	m_NoLanesDirPos = nlpos;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setNoLanesDirNeg(int nlneg)
{
	m_NoLanesDirNeg = nlneg;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setFileIn(CString file)
{
	m_FileIn = file;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setFileOut(CString file)
{
	m_FileOut = file;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setMetricsDir(CString dir)
{
	m_MetricsDir = dir;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setFileType(WORD fileType)
{
	m_FileType = fileType;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setIDMParams_Car(CIDMParameterSet theSet)
{
	m_IDMParams_Car = theSet;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setIDMParams_SmallTruck(CIDMParameterSet theSet)
{
	m_IDMParams_SmallTruck = theSet;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setIDMParams_LargeTruck(CIDMParameterSet theSet)
{
	m_IDMParams_LargeTruck = theSet;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setIDMParams_Crane(CIDMParameterSet theSet)
{
	m_IDMParams_Crane = theSet;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setIDMParams_Lowloader(CIDMParameterSet theSet)
{
	m_IDMParams_Lowloader = theSet;
	SetModifiedFlag();	// To implement Save Changes prompts
}

void CEvolveTrafficDoc::setTrafFileNoLanesDirPos(int nl)
{
	m_TrafFileNoLanesDirPos = nl;
	SetModifiedFlag();
}

void CEvolveTrafficDoc::setTrafFileNoLanesDirNeg(int nl)
{
	m_TrafFileNoLanesDirNeg = nl;
	SetModifiedFlag();
}

void CEvolveTrafficDoc::setLocOutputDetectorDirPos(int loc)
{
	m_LocOutputDetectorDirPos = loc;
	SetModifiedFlag();
}

void CEvolveTrafficDoc::setLocOutputDetectorDirNeg(int loc)
{
	m_LocOutputDetectorDirNeg = loc;
	SetModifiedFlag();
}

void CEvolveTrafficDoc::setAllowLaneChanging(BOOL status)
{
	m_AllowLaneChanging = status;
	SetModifiedFlag();
}

void CEvolveTrafficDoc::setRoadFeatures(CObArray* pRoadFeatures)
{
	m_vRoadFeatures.Copy( *pRoadFeatures );
	SetModifiedFlag();
}

void CEvolveTrafficDoc::setStatDetectors(CObArray* pStatDetectors)
{
	m_vStatDetectors.Copy( *pStatDetectors );
	SetModifiedFlag();
}

void CEvolveTrafficDoc::setDriveOnRight(bool OnRight)
{
	m_DriveOnRight = OnRight;
}
