#ifndef _IDM_H
#define _IDM_H

#include "stdafx.h"
#include "DriverModel.h"

	/** A class representing the IDM driver model*/

class IDM : public DriverModel 
{

public:
	IDM();
	virtual ~IDM();
	double update(double vel, double dist);

	// The gets
	double get_T();
	double get_A();
	double get_B();
	double get_S0();
	double get_S1();
	double get_V0();
	double get_Delta();
	double get_Polite();
	double get_Bias();
	double get_DeltaAth();	

	// The sets
	void set_T		(double val);
	void set_A		(double val);
	void set_B		(double val);
	void set_S0		(double val);
	void set_S1		(double val);
	void set_V0		(double val);
	void set_Delta	(double val);
	void set_Polite	(double val);
	void set_Bias	(double val);
	void set_DeltaAth		(double val);

private:
	// Parameters of model
	/**Safe time headway*/
	double m_T;
	/**Maximum acceleration*/
	double m_A;
	/**Desired deceleration*/
	double m_B;
	/*Jam distance s0*/
	double m_S0;
	/*Jam distance s1*/
	double m_S1;
	/*Desired velocity*/
	double m_V0;
	/*Acceleration exponent*/
	double m_Delta;

	/**MOBIL: Consideration towards other drivers*/
	double m_Polite;
	/**MOBIL: Desire to keep to slow lane*/
	double m_Bias;
	/**MOBIL: Dampens overtaking occurances*/
	double m_DeltaAth;

	double m_V0_BeforeSpeedLimit;	// for storing previous values for retrieval
	double m_T_BeforeGradient;
	double m_A_BeforeGradient;
	double m_B_BeforeGradient;
	double m_V0_BeforeGradient;
	double m_V0_Initial;
	

// Operations
public:
	virtual void ClearRestriction();
	virtual void ClearGradient();
	virtual void SetGradient(double gradient);
	virtual void ClearSpeedLimit();
	virtual void SetSpeedLimit(double newV0);
	virtual double getDesiredVel();			// kept because of base class
	virtual void setVel(double vel);		// set current velocity

	// Lane changing
	double LaneChange(	double GapToFront, 
						double GapToBack, 
						double FrontChangeAccel,
						double CurrentBackAccel, 
						double ProposedBackAccel, 
						bool overtake);

private:
	// properties of vehicle
	double m_speed;
	double s_star(double vel);
	double dvdt(double m_speed, double s);

	double	LANECHANGE_MIN_GAP;
	double	SAFE_BRAKING;
	bool	A_FLAG;
	double	A_MIN;
	bool	B_FLAG;
	double	T_SLOPE;
	double	V0_MIN;
	double	V0_SLOPE;

};
#endif
