// Lane.h: interface for the Lane class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LANE_H__3506F588_71F6_4DF9_AF0A_10B8BA4B9F50__INCLUDED_)
#define AFX_LANE_H__3506F588_71F6_4DF9_AF0A_10B8BA4B9F50__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "stdafx.h"
#include "Vehicle.h"
#include "OutputDetector.h"
#include "SpeedLimit.h"
#include "MetricsDetector.h"
#include "Gradient.h"
#include "LaneChangeDetector.h"

using namespace std;

	/**A class representing a lane in a road*/

class Lane  
{
public:
	void clear();
	void CheckFeatures(const double curTime, double prevPosition, double curPosition, Vehicle* pVeh);
	void setRoadSegments(std::vector<RoadSegment*> segments);
	void setOutputDetector(OutputDetector* out);
	Lane();
	Lane(int iLane, int length, std::vector<Detector*> detectors, bool DirPos, bool AllowLaneChange);
	virtual ~Lane();
	
	bool	update(const double step, const double curTime);
	int		getIndex();
	double	getLastPos();
	int		getNoVeh();
	double	getLength();
	
	void	insert(int i, Vehicle* pVeh);
	void	setRightLane(Lane* right);
	void	setLeftLane(Lane* left);
	
	std::vector<Vehicle*> getPos();	// why return a vector of vehicles?
									// surely should be called getVehicles()

private:
	Vehicle* FinadAdjacentVehicle(Lane* AdjacentLane, double location, 
									bool front, int* iAdjacentLane = NULL);
	
	Lane*	ChangeLanes(Vehicle* pVeh);
	Lane* ImplementLaneChange(Vehicle* pVeh, int LaneChangeID, int iVehLeftLane, int iVehRightLane);

	void CheckDetectors(const double curTime, double prevPosition, double curPosition, Vehicle* pVeh);
	void CheckRoadSegments(double prevPosition, double curPosition, Vehicle* pVeh);	
	
	std::vector<Vehicle*> m_pVehicles;
	std::vector<Detector*> m_vDetectors;

	int		m_Dir;
	int		m_iLane;
	double	m_Flow;
	double	m_RoadLength;
	int		m_NoVeh;
	double	m_PrevVehiclePosition;
	bool	m_AllowLaneChanging;
	bool	m_DirPos;
	double 	m_CurTime;
	bool 	m_bTrackLaneChanges;

	std::vector<RoadSegment*> m_RoadSegments;

	Lane* m_LeftLane;
	Lane* m_RightLane;
	OutputDetector* m_pOutputDetector;
	LaneChangeDetector* m_pLaneChangeDetector;

	double MIN_SPACE_FOR_NEXT_VEHICLE;
};

#endif // !defined(AFX_LANE_H__3506F588_71F6_4DF9_AF0A_10B8BA4B9F50__INCLUDED_)
