/////////////////////////////////////////////////////////////////////////////
// MessageTip.cpp : implementation file
// 
// Author: cunlin zhang
// Date  : 10 February 2001
// Email : zhangcunlin@21cn.com
// Copyright (c) 2001-2003, All Rights Reserved.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then acknowledgement must be made to the author of this file. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
 
#include "stdafx.h"
#include "MessageTip.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

inline static COLORREF PixelAlpha (COLORREF srcPixel, int percent);
inline static COLORREF PixelAlpha (COLORREF srcPixel, COLORREF dstPixel, int percent);
inline static void SetAlphaPixel (COLORREF* pBits, CRect rect, int x, int y, int percent, int iShadowSize);

/////////////////////////////////////////////////////////////////////////////
// Hook Static variables
HWND	CMessageTip::m_hWndReceiver = NULL;
HHOOK	CMessageTip::m_hKeyboardHook = NULL;
HHOOK	CMessageTip::m_hMouseHook = NULL;

/////////////////////////////////////////////////////////////////////////////
// CMessageTip construction
CMessageTip::CMessageTip() :
	m_pParentWnd(NULL),
	m_rcMargin(6, 4, 8, 4),
    m_ptOffset(-6, 0),
    m_nInterval(500),		
	m_nDisplayTime(5000),	//Time to display(by default 5s)
    m_nTimeCount(0),		
	m_iShadowSize(6),
	m_bShadow(TRUE),
	m_bSaveShadows(FALSE),
	m_strTipText("")
{
	m_clrBackColor = ::GetSysColor(COLOR_INFOBK);
    m_clrTextColor = ::GetSysColor(COLOR_INFOTEXT);
}

CMessageTip::~CMessageTip()
{
	Hide();
    m_Font.DeleteObject();
    if ( IsWindow(m_hWnd) )
        DestroyWindow();
}

BEGIN_MESSAGE_MAP(CMessageTip, CWnd)
	//{{AFX_MSG_MAP(CMessageTip)
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_MOUSEACTIVATE()
	ON_WM_SETCURSOR()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMessageTip message handlers
void CMessageTip::OnPaint() 
{
    if (m_strTipText.IsEmpty())
        return;
	CPaintDC dc(this);		// device context for painting

    CRect rect;
    GetClientRect(rect);

	//Draws the shadow for the messagetip
	if (m_bShadow) {
		rect.right -= m_iShadowSize;
		rect.bottom -= m_iShadowSize;
		DrawShadows (dc, rect);
	}

    dc.SetTextColor(m_clrTextColor);

    CFont *pOldFont = dc.SelectObject(&m_Font);
    
	//Draw the border of the messagetip
	dc.FillSolidRect(&rect, m_clrBackColor);
    dc.SelectStockObject(NULL_BRUSH);
    dc.SelectStockObject(BLACK_PEN);
    dc.Rectangle(rect);

	// Draw the messagetip's text
    dc.SetBkMode(TRANSPARENT);
	rect.DeflateRect(m_rcMargin);
    dc.DrawText(m_strTipText, rect, DT_EXPANDTABS|DT_EXTERNALLEADING|DT_NOPREFIX|DT_WORDBREAK);

    dc.SelectObject(pOldFont);

}
/////////////////////////////////////////////////////////////////////////////
//  Timer events are used to remove the message tip after a specified 
//  display time, peridodically check that the mouse is still within
//  the bounds, and check to see if switching to other applications
void CMessageTip::OnTimer(UINT /*nIDEvent*/) 
{

	// switch to other applications - dismiss it 
	if (::GetActiveWindow() == NULL )
	{
        Hide();
        return;
	}

    m_nTimeCount += m_nInterval;
	// check if cursor is in the messagetip window.
    if (IsCursorInTipWindow())
		return;
	if (m_nTimeCount >= m_nDisplayTime )
		Hide();
}
int CMessageTip::OnMouseActivate(CWnd* /*pDesktopWnd*/, UINT /*nHitTest*/, 
									UINT /*message*/)
{
	return MA_NOACTIVATE;
}
BOOL CMessageTip::OnSetCursor(CWnd* /*pWnd*/, UINT /*nHitTest*/, UINT /*message*/) 
{

	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
	return TRUE;
}
void CMessageTip::OnClose() 
{
	Hide();
//	CWnd::OnClose(); don't use here
}

/////////////////////////////////////////////////////////////////////////////
// CMessageTip operations
BOOL CMessageTip::Create(CWnd* pParentWnd)
{

    // Get the class name
    CString strClassName = AfxRegisterWndClass(CS_CLASSDC|CS_SAVEBITS, 
                                              LoadCursor(NULL, IDC_ARROW));

    // Create the window - just don't show it yet
    if (!CWnd::CreateEx(WS_EX_TOOLWINDOW, strClassName, _T(""), 
                        WS_POPUP, 
                        0, 0, 10, 10, // size & position 
                        pParentWnd->GetSafeHwnd(), 0, NULL))
	{
        return FALSE;
	}

    m_pParentWnd = pParentWnd;

	// Get system Font
    NONCLIENTMETRICS ncm;
    ncm.cbSize = sizeof(NONCLIENTMETRICS);
    if (!SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm, 0))
        return FALSE;

    LOGFONT LogFont;
    memcpy(&LogFont, &(ncm.lfStatusFont), sizeof(LOGFONT));

    // Create the actual font object
    m_Font.CreateFontIndirect(&LogFont);

    return TRUE;
   
}
/////////////////////////////////////////////////////////////////////////////
// Hides the messagetip window.
void CMessageTip::Hide()
{
	if ( IsWindow(m_hWnd) )
	{
	    KillTimer(1);
	    ShowWindow(SW_HIDE);
	}
	// remove the hook
	SetHook(m_hWnd, FALSE );
}
/////////////////////////////////////////////////////////////////////////////
// CMessageTip implementation

// [strText]	-	the text to be displayed (may be multiline)
// Get the smallest possible rectangle that will contain 
// the text (inc. margins)
CRect CMessageTip::GetBoundsRect(CString strText) 
{
    CWindowDC dc(NULL);
    CFont *pOldFont = (CFont*) dc.SelectObject((CFont*)&m_Font); 
    int nLineWidth = 0;
 
    // Count the number of lines of text
    int nStart = 0, nNumLines = 0;
	CString strTextCopy = strText;
    do {
        nStart = strTextCopy.Find(_T("\n"));

        // skip found character 
        if (nStart >= 0)
            strTextCopy = strTextCopy.Mid(nStart+1);

        nNumLines++;
    } while (nStart > 0);

    // Find the widest line
    for (int i = 0; i < nNumLines; i++)
    {
        CString strLine = GetFieldFromString(strText, i, _T('\n')) + _T("  ");
        nLineWidth = max(nLineWidth, dc.GetTextExtent(strLine).cx);
    }

    CRect rect(0, 0, max(0, nLineWidth), 0);
    dc.DrawText(strText, rect, DT_CALCRECT | DT_EXPANDTABS|DT_EXTERNALLEADING|DT_NOPREFIX|DT_WORDBREAK);

    rect.bottom += m_rcMargin.top + m_rcMargin.bottom;
    rect.right += m_rcMargin.left + m_rcMargin.right;

	if (m_bShadow)
	{
		rect.right += m_iShadowSize;
		rect.bottom += m_iShadowSize;
	}

    dc.SelectObject(pOldFont);
    return rect;

}
/////////////////////////////////////////////////////////////////////////////
// 	[ref]		-	the string to parse
//	[nIndex]	-	the field number to return (0 based)
//	[ch]		-	the separator character
//  This routine breaks a supplied string into substrings, each delimited
//  by 'ch', and returns the substring number given by nIndex
CString CMessageTip::GetFieldFromString(CString ref, int nIndex, TCHAR ch) const
{
    CString strReturn;
    LPCTSTR pstrStart = ref.LockBuffer();
    LPCTSTR pstrBuffer = pstrStart;
    int nCurrent = 0;
    int nStart = 0;
    int nEnd = 0;
    int nOldStart = 0;

    while (nCurrent <= nIndex && *pstrBuffer != _T('\0'))
    {
        if (*pstrBuffer == ch)
        {
            nOldStart = nStart;
            nStart = nEnd+1;
            nCurrent++;
        }
        nEnd++;
        pstrBuffer++;
    }

    // May have reached the end of the string
    if (*pstrBuffer == _T('\0'))
    {
        nOldStart = nStart;
        nEnd++;
    }

    ref.UnlockBuffer();

    if (nCurrent < nIndex) 
    {
        return strReturn;
    }

    return ref.Mid(nOldStart, nEnd - nOldStart - 1);

}
/////////////////////////////////////////////////////////////////////////////
// [pt]	- the topleft corner of the region
// [rectTextBounds] - the minimum rectangle needed to contain the text ( form GetBoundsRect ) 
// Calculate the bounding rect of some text, this returns the rectangle 
// (in screen coords) that is best suited to display the message
CRect CMessageTip::CalculateInfoBoxRect(CPoint& pt, CRect& rectTextBounds) const
{  

    CRect InfoRect = rectTextBounds;
    InfoRect.OffsetRect(-InfoRect.TopLeft());

    InfoRect.OffsetRect(pt);

    int nHeight = InfoRect.Height();
    InfoRect.top = pt.y - nHeight;
    InfoRect.bottom = InfoRect.top + nHeight;

    // Need to check it'll fit on screen
    CSize ScreenSize(::GetSystemMetrics(SM_CXSCREEN), 
		::GetSystemMetrics(SM_CYSCREEN));

    // Too far right?
    if (InfoRect.right > ScreenSize.cx)
        InfoRect.OffsetRect(-(InfoRect.right - ScreenSize.cx), 0);

    // Too far left?
    if (InfoRect.left < 0)
        InfoRect.OffsetRect( -InfoRect.left, 0);

    if (InfoRect.top < 0)
    {
        InfoRect.OffsetRect( 0, -InfoRect.top);
	}

    // Bottom falling out of screen?
    if (InfoRect.bottom > ScreenSize.cy)
    {

    }
    
    return InfoRect;

}
/////////////////////////////////////////////////////////////////////////////
// [lpszTipText]  -	Pointer to the text
// [lpRect]		  -	Pointer to a RECT structure containing coordinates 
//					of the control's bounding rectangle.
// Diaplays the messagetip 
void CMessageTip::Show(LPCRECT lpRect, LPCTSTR lpszTipText)
{

	if (lpszTipText == NULL || lpRect == NULL || !IsWindow(m_hWnd))
		return;

    m_strTipText = lpszTipText;

	CPoint pt;
	pt = CPoint(lpRect->left, lpRect->top) + (m_bShadow ? m_ptOffset : CPoint(0, -2));

	m_pParentWnd->ClientToScreen(&pt);

    // Set hook
	SetHook(m_hWnd, TRUE);

    //calculate the width and height of the box dynamically
    CRect rect = GetBoundsRect(m_strTipText);
    rect = CalculateInfoBoxRect(pt, rect);

    SetWindowPos(&wndTop, 
                 rect.left, rect.top,
                 rect.Width(), rect.Height(),
                 SWP_SHOWWINDOW|SWP_NOCOPYBITS|SWP_NOACTIVATE|SWP_NOZORDER);

    m_nTimeCount = 0;
    SetTimer(1, m_nInterval, NULL);

}
/////////////////////////////////////////////////////////////////////////////
// Returns TRUE if the cursor is in the messagetip window
BOOL CMessageTip::IsCursorInTipWindow() const
{
    ASSERT(m_pParentWnd);

    if ( !IsWindow(m_hWnd) )
        return FALSE;

    CPoint pt;
    GetCursorPos(&pt);
    CRect rect;
    GetWindowRect(rect);
	if (m_bShadow) {
		rect.right -= m_iShadowSize;
		rect.bottom -= m_iShadowSize;
	}

    return  rect.PtInRect(pt);
}

// ==================================================================
// 
// FUNCTION :  DrawShadows ()
// 
// * Description : Draws the shadow for a rectangular screen element
// 
// * Authors: [Stas Levin ]
//			  [Timo Hummel], Modified: [8/11/99 5:06:59 PM]
//			  
// * Function parameters : 
// [dc] -		The device context to draw into
// [rect] -		The CRect of the rectangular region to draw the
//			    shadow around (altough the CDC needs to be big enough
//				to hold the shadow)
// ==================================================================
void CMessageTip::DrawShadows (CDC& dc, const CRect& rect)
{
	const int iMinBrightness = 100;
	const int iMaxBrightness = 50;

	if (m_iShadowSize == 0)
	{
		return;
	}

	int cx = rect.Width ();
	int cy = rect.Height ();

	if (m_bmpShadowRight.GetSafeHandle () != NULL &&
		m_bmpShadowBottom.GetSafeHandle () != NULL)
	{
		//---------------------------------------------------
		// Shadows are already implemented, put them directly
		// to the DC:
		//---------------------------------------------------
		dc.DrawState (CPoint (rect.right, rect.top), 
					CSize (m_iShadowSize, cy + m_iShadowSize),
					&m_bmpShadowRight, DSS_NORMAL);

		dc.DrawState (CPoint (rect.left, rect.bottom),
					CSize (cx + m_iShadowSize, m_iShadowSize),
					&m_bmpShadowBottom, DSS_NORMAL);
		return;
	}

	ASSERT (m_bmpShadowRight.GetSafeHandle () == NULL);
	ASSERT (m_bmpShadowBottom.GetSafeHandle () == NULL);

	//--------------------------------------------
	// Copy screen content into the memory bitmap:
	//--------------------------------------------
	CDC dcMem;
	if (!dcMem.CreateCompatibleDC (&dc))
	{
		return;
	}

	//--------------------------------------------
	// Gets the whole menu and changes the shadow.
	//--------------------------------------------
	CBitmap	bmpMem;
	if (!bmpMem.CreateCompatibleBitmap (&dc, cx + m_iShadowSize, cy + m_iShadowSize))
	{
		return;
	}

	CBitmap* pOldBmp = dcMem.SelectObject(&bmpMem);
	ASSERT (pOldBmp != NULL);

	LPBITMAPINFO lpbi;

		// Fill in the BITMAPINFOHEADER
	lpbi = (LPBITMAPINFO) new BYTE[sizeof(BITMAPINFOHEADER) ];
	lpbi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	lpbi->bmiHeader.biWidth = cx + m_iShadowSize;
	lpbi->bmiHeader.biHeight = cy + m_iShadowSize;
	lpbi->bmiHeader.biPlanes = 1;
	lpbi->bmiHeader.biBitCount = 32;
	lpbi->bmiHeader.biCompression = BI_RGB;
	lpbi->bmiHeader.biSizeImage = (cx + m_iShadowSize) * (cy + m_iShadowSize);
	lpbi->bmiHeader.biXPelsPerMeter = 0;
	lpbi->bmiHeader.biYPelsPerMeter = 0;
	lpbi->bmiHeader.biClrUsed = 0;
	lpbi->bmiHeader.biClrImportant = 0;

	COLORREF* pBits = NULL;
	HBITMAP hmbpDib = CreateDIBSection (
		dcMem.m_hDC, lpbi, DIB_RGB_COLORS, (void **)&pBits,
		NULL, NULL);

	if (hmbpDib == NULL || pBits == NULL)
	{
		delete lpbi;
		return;
	}

	dcMem.SelectObject (hmbpDib);
	dcMem.BitBlt (0, 0, cx + m_iShadowSize, cy + m_iShadowSize, &dc, rect.left, rect.top, SRCCOPY);

	//----------------------------------------------------------------------------
	// Process shadowing:
	// For having a very nice shadow effect, its actually hard work. Currently,
	// I'm using a more or less "hardcoded" way to set the shadows (by using a
	// hardcoded algorythm):
	//
	// This algorythm works as follows:
	// 
	// It always draws a few lines, from left to bottom, from bottom to right,
	// from right to up, and from up to left). It does this for the specified
	// shadow width and the color settings.
	//-----------------------------------------------------------------------------

	// For speeding up things, iShadowOffset is the
	// value which is needed to multiply for each shadow step
	int iShadowOffset = (iMaxBrightness - iMinBrightness) / m_iShadowSize;

	// Loop for drawing the shadow
	// Actually, this was simpler to implement than I thought
	for (int c = 0; c < m_iShadowSize; c++)
	{
		// Draw the shadow from left to bottom
		for (int y = cy; y < cy + (m_iShadowSize - c); y++)
		{
			SetAlphaPixel (pBits, rect, c + m_iShadowSize, y, 
				iMaxBrightness - ((m_iShadowSize  - c) * (iShadowOffset)),m_iShadowSize);
		}

		// Draw the shadow from left to right
		for (int x = m_iShadowSize + (m_iShadowSize - c); x < cx + c; x++)
		{
			SetAlphaPixel(pBits, rect,x, cy + c,
				iMaxBrightness - ((c) * (iShadowOffset)),m_iShadowSize);
		}

		// Draw the shadow from top to bottom
		for (int y1 = m_iShadowSize + (m_iShadowSize - c); y1 < cy + c + 1; y1++)
		{
			SetAlphaPixel(pBits, rect, cx+c, y1, 
				iMaxBrightness - ((c) * (iShadowOffset)),
				m_iShadowSize);
		}
		
		// Draw the shadow from top to left
		for (int x1 = cx; x1 < cx + (m_iShadowSize - c); x1++)
		{
			SetAlphaPixel (pBits, rect, x1, c + m_iShadowSize,
				iMaxBrightness - ((m_iShadowSize - c) * (iShadowOffset)),
				m_iShadowSize);
		}
	}

	//-----------------------------------------
	// Copy shadowed bitmap back to the screen:
	//-----------------------------------------
	dc.BitBlt (rect.left, rect.top, cx + m_iShadowSize, cy + m_iShadowSize, 
				&dcMem, 0, 0, SRCCOPY);

	if (m_bSaveShadows)
	{
		//------------------------------------
		// Save shadows in the memory bitmaps:
		//------------------------------------
		m_bmpShadowRight.CreateCompatibleBitmap (&dc, m_iShadowSize+1, cy + m_iShadowSize);
		
		dcMem.SelectObject (&m_bmpShadowRight);
		dcMem.BitBlt (0, 0, m_iShadowSize, cy + m_iShadowSize,
						&dc, rect.right, rect.top, SRCCOPY);

		m_bmpShadowBottom.CreateCompatibleBitmap (&dc, cx + m_iShadowSize, m_iShadowSize+1);

		dcMem.SelectObject (&m_bmpShadowBottom);
		dcMem.BitBlt (0, 0, cx + m_iShadowSize, m_iShadowSize,
						&dc, rect.left, rect.bottom, SRCCOPY);
	}

	dcMem.SelectObject (pOldBmp);
	DeleteObject (hmbpDib);
	delete lpbi;
}

// ==================================================================
// 
// FUNCTION :  static void SetAlphaPixel ()
// 
// * Description : Draws an alpha blended pixel
// 
// * Author : [Timo Hummel], Created : [8/11/99 5:04:38 PM]
// 
// * Function parameters : 
// [pBits] -	The DIB bits
// [x] -		X-Coordinate
// [y] -		Y-Coordinate
// [percent] -	Percentage to blit (100 = hollow, 0 = solid)
// 
// ==================================================================
inline static void SetAlphaPixel (COLORREF* pBits, CRect rect, int x, int y, int percent, int m_iShadowSize)
{
	// Our direct bitmap access swapped the y coordinate...
	y = (rect.Height()+m_iShadowSize)- y;

	COLORREF* pColor = (COLORREF*) (pBits + (rect.Width () + m_iShadowSize) * y + x);
	*pColor = PixelAlpha (*pColor, percent);
}

// ==================================================================
// 
// FUNCTION :  PixelAlpha ()
// 
// * Description : Shades a color value with a specified percentage
// 
// * Author : [Timo Hummel], Created : [8/11/99 2:37:04 PM]
// 
// * Returns : [COLORREF] - The result pixel
// 
// * Function parameters : 
// [srcPixel] - The source pixel
// [percent] -  Percentage (amount of shadow)
//
// Example: percent = 10    makes the pixel around 10 times darker
//          percent = 50    makes the pixel around 2 times darker
// 
// ==================================================================
COLORREF PixelAlpha (COLORREF srcPixel, int percent)
{
	// My formula for calculating the transpareny is as
	// follows (for each single color):
	//
	//							   percent
	// destPixel = sourcePixel * ( ------- )
	//                               100
	//
	// This is not real alpha blending, as it only modifies the brightness,
	// but not the color (a real alpha blending had to mix the source and
	// destination pixels, e.g. mixing green and red makes yellow).
	// For our nice "menu" shadows its good enough.

	COLORREF clrFinal = RGB ( (GetRValue (srcPixel) * percent) / 100, 
							  (GetGValue (srcPixel) * percent) / 100, 
							  (GetBValue (srcPixel) * percent) / 100);

	return (clrFinal);

}

// ==================================================================
// 
// FUNCTION :  PixelAlpha ()
// 
// * Description : Shades a color value with a specified percentage
// 
// * Author : [Guillaume Nodet]
// 
// * Returns : [COLORREF] - The result pixel
// 
// * Function parameters : 
// [srcPixel] - The source pixel
// [dstPixel] - The destination pixel
// [percent] -  Percentage (amount of shadow)
//
// ==================================================================
COLORREF PixelAlpha (COLORREF srcPixel, COLORREF dstPixel, int percent)
{
	int ipercent = 100 - percent;
	COLORREF clrFinal = RGB ( (GetRValue (srcPixel) * percent + GetRValue (dstPixel) * ipercent) / 100, 
							  (GetGValue (srcPixel) * percent + GetGValue (dstPixel) * ipercent) / 100, 
							  (GetBValue (srcPixel) * percent + GetBValue (dstPixel) * ipercent) / 100);

	return (clrFinal);

}
// ////////////////////////////////////////////////////////////////////
// [hWnd] Window hook
// [bSetHook] - TRUE sets hook, FALSE removes hook 
// Sets hook or Removes the hook that has been set
void CMessageTip::SetHook(HWND hWnd, BOOL bSetHook)
{
	if (bSetHook)
	{
		if (m_hKeyboardHook == NULL )
			m_hKeyboardHook = ::SetWindowsHookEx( WH_KEYBOARD, 
				( HOOKPROC )KeyboardHookProc, NULL, ::GetCurrentThreadId());
		if (m_hMouseHook == NULL )
			m_hMouseHook = ::SetWindowsHookEx( WH_MOUSE, 
				( HOOKPROC )MouseHookProc, NULL, ::GetCurrentThreadId());
		m_hWndReceiver = hWnd;
	}
	else
	{
	    if( m_hKeyboardHook != NULL)
		{
		    ::UnhookWindowsHookEx( m_hKeyboardHook );
			m_hKeyboardHook = NULL;
		}
	    if( m_hMouseHook != NULL )
		{
		    ::UnhookWindowsHookEx( m_hMouseHook );
			m_hMouseHook = NULL;
		}
		m_hWndReceiver = NULL;
	}
}
// ////////////////////////////////////////////////////////////////////
// intercept keystrokes except Enter Key and PostMessage WM_CLOSE to hide
// the messagetip window
LRESULT CALLBACK CMessageTip::KeyboardHookProc( int nCode, WPARAM wParam, LPARAM lParam )
{
	if ( nCode == HC_ACTION )
	{
        if ( !( HIWORD( lParam ) & KF_UP ) && wParam != VK_RETURN)
			::PostMessage(m_hWndReceiver, WM_CLOSE, 0, 0); // dismiss it
	}
	return CallNextHookEx( m_hKeyboardHook, nCode, wParam, lParam );
}
// ////////////////////////////////////////////////////////////////////
// intercept most mouse down event and PostMessage WM_CLOSE to hide 
// the messagetip window
LRESULT CALLBACK CMessageTip::MouseHookProc( int nCode, WPARAM wParam, LPARAM lParam )
{
	if ( nCode == HC_ACTION )
	{
	    if ( wParam == WM_LBUTTONDOWN   || wParam == WM_RBUTTONDOWN   ||
             wParam == WM_NCLBUTTONDOWN || wParam == WM_NCRBUTTONDOWN || 
             wParam == WM_MBUTTONDOWN   || wParam == WM_NCMBUTTONDOWN )
		{
			::PostMessage(m_hWndReceiver, WM_CLOSE, 0, 0); // dismiss it
		}
	}
	return CallNextHookEx( m_hMouseHook, nCode, wParam, lParam );
}
