// OutputDetector.cpp: implementation of the OutputDetector class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "OutputDetector.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

	/** Constructor
	 *
	 * @param loc The location of the OutputDetector
	 * @param DirPos Whether or not the OutputDetector is in the positive direction
	 * @param fh The filehandler to use
	 */
OutputDetector::OutputDetector(int loc, bool DirPos, FileHandler* fh)
{
	m_Location = loc;
	m_DirPos = DirPos;
	m_pFileHandler = fh;
}

	/** Default Desctructor*/
OutputDetector::~OutputDetector()
{

}

	/** Adds a vehicle to the OutputDetector
	 *
	 * @param pVeh The vehicle to add
	 * @param curTime The current simulation time
	 */
void OutputDetector::addVehicle(Vehicle *pVeh, double curTime)
{
	// set the vehicle time to the arrival time at the detector
	double ArrivalTime = FindDetectorArrivalTime(pVeh, curTime);
	pVeh->setTime(ArrivalTime);
	// now add to vector prior to writing
	m_vVehicles.push_back(pVeh);
}

	/** Writes the OutputDetector's vehicles to file*/
void OutputDetector::WriteVehiclesToFile()
{
	for(int i = 0; i < m_vVehicles.size(); i++)
		m_pFileHandler->writeLine(m_vVehicles.at(i));
	// once written, they must be deleted
	m_vVehicles.clear();
}

	/**Sets the OutputDetector's filehandler
	 *
	 * @param fh The filehandler to use
	 */
void OutputDetector::setFileHandler(FileHandler *fh)
{
	m_pFileHandler = fh;
}

void OutputDetector::clear()
{
	m_vVehicles.clear();
}
