// Parameter.cpp : implementation file
//

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "Parameter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CParameter

IMPLEMENT_SERIAL(CParameter, CObject, VERSION_NUMBER)

CParameter::CParameter()
{

}

CParameter::~CParameter()
{

}

CParameter::CParameter(const CParameter& param)
{
	this->m_Distribution = param.m_Distribution;
	this->m_ID = param.getID();
}

/////////////////////////////////////////////////////////////////////////////
// CParameter diagnostics

#ifdef _DEBUG
void CParameter::AssertValid() const
{
	CObject::AssertValid();
}

void CParameter::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CParameter serialization

void CParameter::Serialize(CArchive& ar)
{
	m_Distribution.Serialize(ar);
	if (ar.IsStoring())
		ar << m_ID;
	else
		ar >> m_ID;
}

/////////////////////////////////////////////////////////////////////////////
// CParameter message handlers

void CParameter::SetDefaultParams()
{
	// Here we set the default IDM properties based on ID
	// No set shapes since default distribvution is DIST_NORMAL
	switch(m_ID)
	{
		case IDM_PARAM_T:
			{
				m_Distribution.setLocation(1.2);
				m_Distribution.setScale(0.05);
			}
			break;
		case IDM_PARAM_A:
			{
				m_Distribution.setLocation(1.0);
				m_Distribution.setScale(0.05);			
			}
			break;
		case IDM_PARAM_B:
			{
				m_Distribution.setLocation(2.0);
				m_Distribution.setScale(0.1);			
			}
			break;
		case IDM_PARAM_S0:
			{
				m_Distribution.setLocation(1.0);
				m_Distribution.setScale(0.05);			
			}
			break;
		case IDM_PARAM_S1:
			{
				m_Distribution.setLocation(10.0);
				m_Distribution.setScale(0.7);			
			}
			break;
		case IDM_PARAM_V0:
			{	// Defined as km/h
				m_Distribution.setLocation(80);
				m_Distribution.setScale(3.0);			
			}
			break;
		case IDM_PARAM_DELTA:
			{	// Leave as a constant
				m_Distribution.setDistributionID(DIST_CONST);
				m_Distribution.setLocation(4);
			}
			break;
		case IDM_PARAM_POLITE:
			{	// Leave as a constant
				m_Distribution.setDistributionID(DIST_CONST);
				m_Distribution.setLocation(0.0);
			}
			break;
		case IDM_PARAM_BIAS:
			{	// Leave as a constant
				m_Distribution.setDistributionID(DIST_CONST);
				m_Distribution.setLocation(0.1);
			}
			break;
		case IDM_PARAM_DELTAATH:
			{	// Leave as a constant
				m_Distribution.setDistributionID(DIST_CONST);
				m_Distribution.setLocation(0.3);
			}
			break;
		default: // IDM_PARAM_V0:
			{	// Defined as km/h
				m_Distribution.setLocation(80);
				m_Distribution.setScale(3.0);			
			}
			break;
	}
}

CParameter& CParameter::operator=(const CParameter &param)
{
	this->m_Distribution = param.m_Distribution;
	this->m_ID = param.getID();

	return *this;
}

CDistribution* CParameter::getDistribution()
{
	return &m_Distribution;
}

void CParameter::setDistribution(CDistribution* dist)
{
	m_Distribution = *dist;
}

WORD CParameter::getID() const
{
	return m_ID;
}

void CParameter::setID(WORD id)
{
	m_ID = id;
	SetDefaultParams();
}

// This function returns generated value for the parameter
double CParameter::Generate()
{
	return m_Distribution.Generate();
}

double CParameter::GenKeepingCoV(double newLocation)
{
	return m_Distribution.GenKeepingCoV(newLocation);
}

