// PreferencesDlg.cpp : implementation file
//

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "PreferencesDlg.h"

#include "GridCtrl_src/GridCellNumeric.h"

#include "ConfigData.h"
extern CConfigData g_ConfigData;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPreferencesDlg dialog


CPreferencesDlg::CPreferencesDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPreferencesDlg::IDD, pParent)
{
	MIN_VIEW_SCALE = g_ConfigData.View.MIN_VIEW_SCALE;

	m_sRowHeads.Add("Overall scale (meters/pixel)");
	m_sRowHeads.Add("Widths Scale");
	m_sRowHeads.Add("Time warp factor");
	m_sRowHeads.Add("Lane width (m)");
	m_sRowHeads.Add("Vehicle width (m)");
	m_sRowHeads.Add("Vehicle Length Scale");
	m_sRowHeads.Add("Show Velocities");
	m_sRowHeads.Add("Show Legend");
	m_sRowHeads.Add("Tick Length (m)");
	m_sRowHeads.Add("Tick Interval (m)");
	m_sRowHeads.Add("Border Top (m)");
	m_sRowHeads.Add("Border Bottom (m)");
	m_sRowHeads.Add("Border Left (m)");
	m_sRowHeads.Add("Border Right (m)");

	m_sColHeads.Add("Setting");
	m_sColHeads.Add("Value");
	
	m_nFixRows = FIXED_ROWS;	// MAGIC NUMBER - row and column headers
	m_nFixCols = FIXED_COLUMNS;
	
	m_nCols = 1 + m_nFixCols;	// MAGIC NUMBER - obviously 1 extra column
	m_nRows = m_sRowHeads.GetSize() + m_nFixRows;

	//{{AFX_DATA_INIT(CPreferencesDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CPreferencesDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPreferencesDlg)
	DDX_Control(pDX, IDC_GRID, m_Grid);             // associate the grid window with a C++ object
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPreferencesDlg, CDialog)
	//{{AFX_MSG_MAP(CPreferencesDlg)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_GRID, OnGridEndEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPreferencesDlg message handlers


void CPreferencesDlg::OnOK() 
{
	CDialog::OnOK();
	if(m_Scale < MIN_VIEW_SCALE)
	{
		MessageBox("View scale cannot be less than minimum - default minimum set", "EvolveTraffic", MB_OK | MB_ICONWARNING);
		m_Scale = MIN_VIEW_SCALE;
	}
	if(m_ExaggerateWidths < 1.0)
	{
		MessageBox("Widths scale cannot be less than 1.0 - default set", "EvolveTraffic", MB_OK | MB_ICONWARNING);
		m_ExaggerateWidths = 1.0;
	}
	if(m_TimeWarp < 1.0)
	{
		MessageBox("Widths scale cannot be less than 1.0 - default set", "EvolveTraffic", MB_OK | MB_ICONWARNING);
		m_ExaggerateWidths = 1.0;
	}
	if(m_VehicleWidth > m_LaneWidth)
	{
		MessageBox("Vehicle width should be less than lane width - default of 80% of lane width set", "EvolveTraffic",	MB_OK | MB_ICONWARNING);
		m_VehicleWidth = 0.8 * m_LaneWidth;	// MAGIC NUMBER - no harm having a magic number here
	}
}

BOOL CPreferencesDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	TRY {
		m_Grid.SetRowCount(m_nRows);
		m_Grid.SetColumnCount(m_nCols);
		m_Grid.SetFixedRowCount(m_nFixRows);
		m_Grid.SetFixedColumnCount(m_nFixCols);
	}
	CATCH (CMemoryException, e)
	{
		e->ReportError();
		return FALSE;
	}
	END_CATCH
	
	SetGridHeadings();
	SetCells();
	m_Grid.ExpandToFit();	
	
	LoadSettingsIntoGrid();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CPreferencesDlg::SetGridHeadings()
{
	int row = 0;
	int col = 0;

	// Set fixed column text
	m_Grid.SetItemText(row,m_nFixCols-1,m_sColHeads.GetAt(0));
	m_Grid.SetItemText(row,m_nFixCols,m_sColHeads.GetAt(1));

	// Set fixed row text
	col = 0;
	for (row = 1; row < m_Grid.GetRowCount(); row++)
		m_Grid.SetItemText(row,col,m_sRowHeads.GetAt(row-1));
}

void CPreferencesDlg::SetCells()
{
	for (int row = 1; row < m_nRows; row++)
	{
		int col = 1;
		m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellNumeric));
		m_Grid.GetCell(row,col)->SetFormat(DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_NOPREFIX|DT_END_ELLIPSIS);
		CGridCellNumeric *pCell = (CGridCellNumeric*)m_Grid.GetCell(row,col);
		pCell->SetFlags(CGridCellNumeric::Real);	// no negative numbers
	}
}

void CPreferencesDlg::LoadSettingsIntoGrid()
{
	int col = 1;
	for (int row = 1; row < m_nRows; row++)
	{
		CGridCellNumeric* pCell = (CGridCellNumeric *)(m_Grid.GetCell(row,col));
		pCell->SetNumber( *(MapRowToValue(row)) );
	}
}

void CPreferencesDlg::OnGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult)
{
	// if change is ok, then *pResult 0, else *pResult -1
	
	// pItem is the cell that has just been edited
    NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
    
    int row = pItem->iRow;
	int col = pItem->iColumn;

	double val = ((CGridCellNumeric *)(m_Grid.GetCell(row,col)))->GetNumber();
	*(MapRowToValue(row)) = val;

	*pResult = 0;
}

double* CPreferencesDlg::MapRowToValue(int row)
{
	// This functions assumes the variable order as per m_sRowHeads

	row = row - m_nFixRows;	// keeps independence of m_nFixRows
	switch(row)
	{
		case 0:
			return &m_Scale;
		case 1:
			return &m_ExaggerateWidths;
		case 2:
			return &m_TimeWarp;
		case 3:
			return &m_LaneWidth;
		case 4:
			return &m_VehicleWidth;
		case 5:
			return &m_VehicleLengthScale;
		case 6:
			return &m_ShowVelocity;
		case 7:
			return &m_ShowLegend;
		case 8:
			return &m_TickLength;
		case 9:
			return &m_TickStep;
		case 10:
			return &m_Border_Top;
		case 11:
			return &m_Border_Btm;
		case 12:
			return &m_Border_Lhs;
		case 13:
			return &m_Border_Rhs;
		default:
			return &m_Border_Top;
	}
}
