#ifndef _ROAD_H
#define _ROAD_H

#include "stdafx.h"
#include "IDMParameterSet.h"
#include "RoadFeature.h"
#include "StatDetector.h"
#include "Direction.h"

#include "CASTORFile.h"
#include "SAFTFile.h"
#include "Constants.h"
#include <vector>
#include <fstream>
#include <algorithm>

typedef std::vector< std::vector<Vehicle*> > M2D;

	/** A class to represent a road in the simulation */

class Road {

public:
	void setLaneChangeReg(bool trackChanges, double outputTime, int roadLen, WORD registerType);
	void setDriveOnRight(bool OnRight);
	void clear();
	int	getPercentComplete();
	bool getAllowLaneChanging();
	
	void setAllowLaneChanging(bool status);
	void setLocOuputDetectorDirPos(int loc);
	void setLocOuputDetectorDirNeg(int loc);
	
	
	void setNoLanesDirPos(int nlpos);
	void setNoLanesDirNeg(int nlneg);
	void setNoDirections(int nd);
	void setNoLanes(int nl);
	void setRoadLength(int L);
	void setTrafFileNoLanesDirPos(int nl);
	void setTrafFileNoLanesDirNeg(int nl);

	void setIDMParams_Car		(CIDMParameterSet* Params);
	void setIDMParams_SmallTruck(CIDMParameterSet* Params);
	void setIDMParams_LargeTruck(CIDMParameterSet* Params);
	void setIDMParams_Crane		(CIDMParameterSet* Params);
	void setIDMParams_Lowloader	(CIDMParameterSet* Params);
	
	void SetSegmentFromFeature(CRoadFeature* pFeat);
	void SetMetricDetFromStatDet(CStatDetector* pDet, int nLanesPos, int nLanesNeg);

	double getLength();
	M2D getVehicles();
	
	void initTruckGroup(string inFile, string outFile, WORD fileType);
	bool update(double step, const double curTime);
	void populate(double step, const double curTime);
	double init();

	Road();
	virtual ~Road();

private:
	bool m_DriveOnRight;
	int m_PercentComplete;
	void passIDMParameters(RoadSegment* segment);
	void MapTrafLaneToSimLane(Vehicle* pVeh);
	void setIDMDriverModel(Vehicle* pVeh);
	void SetSpeedLimitSegment(CRoadFeature* pFeat);
	void SetGradientSegment(CRoadFeature *pFeat);

	CIDMParameterSet* m_pIDMParams_Car;
	CIDMParameterSet* m_pIDMParams_SmallTruck;
	CIDMParameterSet* m_pIDMParams_LargeTruck;
	CIDMParameterSet* m_pIDMParams_Crane;
	CIDMParameterSet* m_pIDMParams_Lowloader;
	
	int m_LocOutputDetectorDirPos;
	int m_LocOutputDetectorDirNeg;
	
	Direction DirectionPos;
	Direction DirectionNeg;

	int m_TrafFileNoLanesDirPos;
	int m_TrafFileNoLanesDirNeg;
	
	FileHandler* m_pFileHandler;
	std::vector<Vehicle*>	m_BufferVehicles;
	std::vector<Lane*>		m_vLanes;
	std::vector<Detector*>	m_vDetectorsPos;
	std::vector<Detector*>	m_vDetectorsNeg;

	std::vector<int>		m_vLaneLengthsPos;
	std::vector<int>		m_vLaneLengthsNeg;
	
	std::vector<RoadSegment*> m_vRoadSegmentsPos;
	std::vector<RoadSegment*> m_vRoadSegmentsNeg;

	bool	m_bEndOfFile;
	bool	m_AllowLaneChanging;
	int		m_NoLanesDirPos;
	int		m_NoLanesDirNeg;
	int		m_NoDirections;
	int		m_NoLanes;
	int		m_RoadLength;

	double	MIN_SPACE_FOR_NEXT_VEHICLE;

};
#endif
