// SAFTFile.cpp: implementation of the SAFTFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "SAFTFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

	/** Constructor
	 *
	 * @param inputFile The file to read input from
	 * @param outputFile The file to write output to
	 *
	 */
SAFTFile::SAFTFile(std::string inputFile, std::string outputFile)
{
	m_InFile.open(inputFile.c_str(), std::ios::in);
	setInFileSize();

	m_OutFile.open(outputFile.c_str(), std::ios::out);
	m_pCharVeh = m_VehCharArry;
}

	/** Default destructor */
SAFTFile::~SAFTFile()
{
	m_InFile.close();
	m_OutFile.flush();

	m_Vehicles.clear();
}

	/** Reads a line representing a truck from a SAFT file
	 *
	 * @return A newly created truck
	 * 
	 * This function reads the next line available from a SAFT format file
	 * and stores the information in a string. If the string is not null, a
	 * truck is created with the properties specified by the line from the
	 * file, and the truck is returned
	 */
Vehicle* SAFTFile::readLine()
{
	std::string str;

	std::getline(m_InFile, str, '\n');

	if(str != "")
	{
		m_CurLine++;
		Vehicle* pVeh;
		double GVW = atoi( str.substr(27,4).c_str() ) * KG100_TO_KN;	// convert kg/100 to kN

		if(GVW >= TRUCK_WEIGHT_THRESHOLD)
			pVeh = new Truck();

		else
			pVeh = new Car();

		pVeh->createSAFTVehicle(str);
		return pVeh;
	}

	return NULL;
}

	/** Writes a line representing a truck to a SAFT file
	 *
	 * @param pVeh The vehicle to write to file
	 *
	 * This function takes a vehicle as a parameter, and calls this vehicle's
	 * doSAFTDATA function, in order to obtain a string which represents all
	 * of the vehicle's physical properties. It then writes this string to a 
	 * SAFT format file
	 */
void SAFTFile::writeLine(Vehicle *pVeh)
{
	pVeh->writeSAFTData(m_pCharVeh);
	m_OutFile << m_pCharVeh << '\n';
}
