// SpeedLimit.cpp: implementation of the SpeedLimit class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SpeedLimit.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

	/**Constructor*/
SpeedLimit::SpeedLimit(int start, int end, double limit, bool DirPos)
{
	m_Beginning		= start;
	m_End			= end;
	m_DirPos		= DirPos;
	m_SpeedLimit	= limit;
}

	/**Destructor*/
SpeedLimit::~SpeedLimit()
{

}

double SpeedLimit::getSpeedLimit()
{
	return m_SpeedLimit;
}

	/**Adds a vehicle to the speed limit
	 *
	 * @param pVeh The vehicle to add
	 * 
	 * This function adds a vehicle to a speed limit segment in the road. A new desired
	 * acceleration for the vehicle is created based on the type of vehicle. If, however,
	 * the vehicle's desired acceleration is below that of the speed limit, then the 
	 * desired acceleration does not change
	 */
void SpeedLimit::addVehicle(Vehicle *pVeh)
{
	double curV0 = pVeh->getDesiredVel();
	double newV0 = curV0;	// set limit to current V0 if curV0 < limit

	// do we need to generate a new lower V0?
	if(curV0 > m_SpeedLimit)
	{
		WORD veh_id = pVeh->getID();
		CIDMParameterSet* paramSet = getIDMParams(veh_id);
		CParameter V0 = (*paramSet->get_V0());
		// Initial method - generate new V0
		//newV0 = V0.GenKeepingCoV(m_SpeedLimit)*KM_PER_H_TO_M_PER_S;
		// revised method - keep ratio of v0/location constant
		double meanV0 = (V0.getDistribution()->getLocation()*KM_PER_H_TO_M_PER_S);
		double v0ratio = curV0/meanV0;
		newV0 = v0ratio * m_SpeedLimit;
		//TRACE("***Speed limit - m_SpeedLimit: %f\tcurV0: %f\tmeanV0: %f\tv0ratio: %f\tnewV0: %f\n",m_SpeedLimit, curV0, meanV0, v0ratio, newV0);
	}
	// this method allows for other driver models
	DriverModel* driver = pVeh->getDriver();
	driver->SetSpeedLimit(newV0);	// always call this function, even if not changing V0
}

	/**Removes a vehicle from a speed limit
	 *
	 * @param pVeh The Vehicle to remove
	 */
void SpeedLimit::removeVehicle(Vehicle *pVeh)
{
	DriverModel* driver = pVeh->getDriver();
	driver->ClearSpeedLimit();
}
