// StatDetectorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "StatDetectorDlg.h"

#include "GridCtrl_src\GridCellCombo.h"
#include "GridCtrl_src\GridCellNumeric.h"

#include "StatDetector.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CStatDetectorDlg dialog


CStatDetectorDlg::CStatDetectorDlg(CWnd* pParent /*=NULL*/)
	: CDialogExt(CStatDetectorDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CStatDetectorDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_sStatDetectorTypes.Add("Flow & Density");
	m_sStatDetectorTypes.Add("Headway");
	m_sStatDetectorTypes.Add("Composition");
	m_sStatDetectorTypes.Add("Lane Changes");
	
	m_sDirections.Add("Positive x");
	m_sDirections.Add("Negative x");

	m_sVehicleTypes.Add("All Vehicles");
	m_sVehicleTypes.Add("Cars");
	m_sVehicleTypes.Add("Small Trucks");
	m_sVehicleTypes.Add("Large Trucks");
	m_sVehicleTypes.Add("Cranes");
	m_sVehicleTypes.Add("Low-loaders");

	m_sColumnHeaders.Add("Type");
	m_sColumnHeaders.Add("Direction");
	m_sColumnHeaders.Add("Vehicle Types");
	m_sColumnHeaders.Add("Time Interval (s)");
	m_sColumnHeaders.Add("Location / Step (m)");

	m_nFixCols = FIXED_COLUMNS;
	m_nFixRows = FIXED_ROWS;	
	m_nCols = m_sColumnHeaders.GetSize() + m_nFixCols;
}


void CStatDetectorDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogExt::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CStatDetectorDlg)
	DDX_Control(pDX, IDC_GRID, m_Grid);             // associate the grid window with a C++ object
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CStatDetectorDlg, CDialogExt)
	//{{AFX_MSG_MAP(CStatDetectorDlg)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_GRID, OnGridEndEdit)
	ON_BN_CLICKED(IDC_BTN_ADDDET, OnBtnAdddet)
	ON_BN_CLICKED(IDC_BTN_DELDET, OnBtnDeldet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStatDetectorDlg message handlers

BOOL CStatDetectorDlg::OnInitDialog()
{
	CDialogExt::OnInitDialog();

	// init rows
	m_NoStatDetectors = m_vStatDetectors.GetSize();
	m_nRows = m_nFixRows + m_NoStatDetectors;

	TRY {
		m_Grid.SetRowCount(m_nRows);
		m_Grid.SetColumnCount(m_nCols);
		m_Grid.SetFixedRowCount(m_nFixRows);
		m_Grid.SetFixedColumnCount(m_nFixCols);
	}
	CATCH (CMemoryException, e)
	{
		e->ReportError();
		return FALSE;
	}
	END_CATCH
	
	SetGridHeadings();
	SetCells(false);	// false because not addition or detletion
	m_Grid.ExpandColumnsToFit();	
	
	LoadStatDetectorsIntoGrid();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CStatDetectorDlg::OnValidate(UINT &nID, CString &strMessage)
{
	int nSD = m_vStatDetectors.GetSize();
	
	for(int i = 0; i < nSD;  i++)
	{
		CStatDetector* pSD = reinterpret_cast<CStatDetector*>(m_vStatDetectors.GetAt(i));

		int location = pSD->getLocation();
		if(location > m_RoadLength)	// since negative numbers can't occur - only check the end
		{
			strMessage.Format("Detector %d: must be located on the road.\nPlease check its location.", i+1);
			nID = IDC_GRID;
			return;
		}

		int time = pSD->getTimeInterval();
		if(time < 10 && time > 0)
		{
			CString str; str.Format("Detector %d: The time interval is very small and may cause problems.", i+1);
			int result = MessageBox(str + "\nAre you sure you want to keep this value?","EvolveTraffic", MB_YESNO | MB_ICONWARNING);
			if(result == IDNO)
			{
				strMessage.Format("Detector %d: Change the time interval.", i+1);
				nID = IDC_GRID;
				return;
			}
		}
		else if(time == 0)
		{
			strMessage.Format("Detector %d: The time interval must not be zero.", i+1);
			nID = IDC_GRID;
			return;
		}

		WORD DetType = pSD->getDetectorType();
		int dist_interval = location;
		if(DetType == METRICS_TYPE_LANE_CHANGE && dist_interval > 1000)
		{
			CString str; str.Format("Detector %d: The lane change detector step is very large and may cause problems.", i+1);
			MessageBox(str,"EvolveTraffic", MB_OK | MB_ICONWARNING);
		}
	}
}

void CStatDetectorDlg::SetGridHeadings()
{
	int row = 0;
	int col = 0;

	// Set fixed column text
	for (col = 1; col < m_nCols; col++)
		m_Grid.SetItemText(row,col,m_sColumnHeaders.GetAt(col-1));

	// Set fixed row text
	col = 0;
	for(row = m_nFixRows; row < m_nRows; row++)
	{
		CString str;
		str.Format("Detector %d", row);
		m_Grid.SetItemText(row,col,str);
	}
}

void CStatDetectorDlg::SetCells(bool bAddDelete)
{
	// these lines are for the resizing of the rows due to addtions/deletions
	if(bAddDelete)
	{
		m_NoStatDetectors = m_vStatDetectors.GetSize();
		m_nRows = m_nFixRows + m_NoStatDetectors;
		m_Grid.SetRowCount(m_nRows);
		// update row headings
		for(int row = m_nFixRows; row < m_nRows; row++)
		{
			CString str;
			str.Format("Detector %d", row);
			m_Grid.SetItemText(row,0,str);
		}
	}

	for (int row = m_nFixRows; row < m_nRows; row++)
	{
		int col = 1;	// Do Detector-type drop down
		m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellCombo));	
	    CGridCellCombo *pCell = (CGridCellCombo*) m_Grid.GetCell(row,col);
		pCell->SetOptions(m_sStatDetectorTypes);
		pCell->SetStyle(CBS_DROPDOWNLIST);

		col = 2;		// Do directions drop down
		m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellCombo));	
	    pCell = (CGridCellCombo*) m_Grid.GetCell(row,col);
		pCell->SetOptions(m_sDirections);
		pCell->SetStyle(CBS_DROPDOWNLIST);

		col = 3;		// Do vehicles drop down
		m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellCombo));	
	    pCell = (CGridCellCombo*) m_Grid.GetCell(row,col);
		pCell->SetOptions(m_sVehicleTypes);
		pCell->SetStyle(CBS_DROPDOWNLIST);

		col = 4;		// Do time interval cells
		m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellNumeric));
		m_Grid.GetCell(row,col)->SetFormat(DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_NOPREFIX|DT_END_ELLIPSIS);
		CGridCellNumeric *pCellN = (CGridCellNumeric*)m_Grid.GetCell(row,col);
		pCellN->SetFlags(CGridCellNumeric::Integer);

		col = 5;		// Do location cells
		m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellNumeric));
		m_Grid.GetCell(row,col)->SetFormat(DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_NOPREFIX|DT_END_ELLIPSIS);
		pCellN = (CGridCellNumeric*)m_Grid.GetCell(row,col);
		pCellN->SetFlags(CGridCellNumeric::Integer);
	}
}

void CStatDetectorDlg::LoadStatDetectorsIntoGrid()
{
	for(int i = 0; i < m_NoStatDetectors; i++)
	{
		int col = m_nFixCols;
		int row = m_nFixRows + i;
		CStatDetector* pStatDet = reinterpret_cast<CStatDetector*>(m_vStatDetectors[i]);

		CString txt = MapDetTypeToString( pStatDet->getDetectorType() );
		m_Grid.SetItemText(row,col,txt);

		col++;	// next col - set direction text
		txt = pStatDet->getDirPos() == true ? m_sDirections.GetAt(0) : m_sDirections.GetAt(1);
		m_Grid.SetItemText(row,col,txt);

		col++;	// next col - set vehicle text
		txt = MapVehTypeToString( pStatDet->getVehicleType() );
		m_Grid.SetItemText(row,col,txt);

		col++;	// next col - set time interval
		CGridCellNumeric* pCell = (CGridCellNumeric *)(m_Grid.GetCell(row,col));
		pCell->SetNumber( pStatDet->getTimeInterval() );

		col++;	// next col - set location
		pCell = (CGridCellNumeric *)(m_Grid.GetCell(row,col));
		pCell->SetNumber( pStatDet->getLocation() );
	}
	m_Grid.AutoSizeRows();
	m_Grid.Invalidate();
}

void CStatDetectorDlg::OnBtnAdddet() 
{
	// Add the feature to the vector
	m_NoStatDetectors++;
	CStatDetector* pStatDet = new CStatDetector;	// create one to use
	m_vStatDetectors.Add(pStatDet);

	// add a row to the table
	SetCells(true);
	LoadStatDetectorsIntoGrid();		
}

void CStatDetectorDlg::OnBtnDeldet() 
{
	if(m_NoStatDetectors != 0)
	{
		// Get the ptr and remove from the vector
		CStatDetector* pStatDet = reinterpret_cast<CStatDetector*>(m_vStatDetectors.GetAt(m_NoStatDetectors-1));
		m_vStatDetectors.RemoveAt( m_NoStatDetectors-1 );

		delete pStatDet;	// remove feature from heap
		m_NoStatDetectors--;	// decrement only now

		// update the table
		SetCells(true);
		LoadStatDetectorsIntoGrid();
	}
	else
		MessageBox("No statistics detector to delete.", "EvolveTraffic", MB_OK|MB_ICONWARNING);
}

// This function handles the GVN_ENDLABELEDIT messages
void CStatDetectorDlg::OnGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult)
{
	// if change is ok, then *pResult 0, else *pResult -1
	
	// pItem is the cell that has just been edited
    NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
    
    int row = pItem->iRow;	int col = pItem->iColumn;

	if(col < 3 + m_nFixRows )	// is the type, direction, or vehicle column
	{
		CString strCell = m_Grid.GetCell(row,col)->GetText();
		if(strCell != "")
		{
			// data is ok
			SetParamData(row,col,strCell);
			*pResult = 0;
		}	
		else
			*pResult = -1;
	}
	else
	{
		int val = (int)((CGridCellNumeric *)(m_Grid.GetCell(row,col)))->GetNumber();
		SetParamData(row,col,val);
		*pResult = 0;
	}

}

void CStatDetectorDlg::SetParamData(int row, int col, CString str)
{
	// if it's a string it's either col 1 or 2
	CStatDetector* pStatDet = reinterpret_cast<CStatDetector*>(m_vStatDetectors[row-m_nFixRows]);
	
	col = col - m_nFixCols;
	

	switch(col)
	{
		case 0:			// detector type
			pStatDet->setDetectorType( MapStringToDetType(str) );
			break;
		case 1:			// direction
			{
				bool DirPos = str == m_sDirections.GetAt(0) ? true : false;
				pStatDet->setDirPos(DirPos);
				break;
			}
		default:		// vehicle
			pStatDet->setVehicleType( MapStringToVehType(str) );
	}
}

void CStatDetectorDlg::SetParamData(int row, int col, int val)
{
	CStatDetector* pStatDet = reinterpret_cast<CStatDetector*>(m_vStatDetectors[row-m_nFixRows]);

	col = col - m_nFixCols;

	switch(col)
	{
		case 3:		// time
			pStatDet->setTimeInterval(val);
			break;
		default:	// location
			pStatDet->setLocation(val);
	}
}

CString CStatDetectorDlg::MapVehTypeToString(WORD type)
{
	switch(type)
	{
		case METRICS_VEH_ALL:			return m_sVehicleTypes.GetAt(0);
		case METRICS_VEH_CAR:			return m_sVehicleTypes.GetAt(1);
		case METRICS_VEH_SMALLTRUCK:	return m_sVehicleTypes.GetAt(2);
		case METRICS_VEH_LARGETRUCK:	return m_sVehicleTypes.GetAt(3);
		case METRICS_VEH_CRANE:			return m_sVehicleTypes.GetAt(4);
		case METRICS_VEH_LOWLOADER:		return m_sVehicleTypes.GetAt(5);
		default:						return m_sVehicleTypes.GetAt(0);	// all vehicles
	}
}

CString CStatDetectorDlg::MapDetTypeToString(WORD type)
{
	switch(type)
	{
		case METRICS_TYPE_FLOWDENSITY:	return m_sStatDetectorTypes.GetAt(0);
		case METRICS_TYPE_HEADWAY:		return m_sStatDetectorTypes.GetAt(1);
		case METRICS_TYPE_COMPOSITION:	return m_sStatDetectorTypes.GetAt(2);
		case METRICS_TYPE_LANE_CHANGE:	return m_sStatDetectorTypes.GetAt(3);
		default:						return m_sStatDetectorTypes.GetAt(0);	// all vehicles
	}
}

WORD CStatDetectorDlg::MapStringToVehType(CString str)
{
	// Note this function reqires the order of the types 
	// to not change
	int i = 0;
	while(str != m_sVehicleTypes.GetAt(i))
		i++;

	switch(i)
	{
		case 0:		return METRICS_VEH_ALL;
		case 1:		return METRICS_VEH_CAR;
		case 2:		return METRICS_VEH_SMALLTRUCK;
		case 3:		return METRICS_VEH_LARGETRUCK;
		case 4:		return METRICS_VEH_CRANE;
		case 5:		return METRICS_VEH_LOWLOADER;
		default:	return METRICS_VEH_ALL;
	}
}

WORD CStatDetectorDlg::MapStringToDetType(CString str)
{
	// Note this function reqires the order of the types 
	// to not change
	int i = 0;
	while(str != m_sStatDetectorTypes.GetAt(i))
		i++;

	switch(i)
	{
		case 0:		return METRICS_TYPE_FLOWDENSITY;
		case 1:		return METRICS_TYPE_HEADWAY;
		case 2:		return METRICS_TYPE_COMPOSITION;
		case 3:		return METRICS_TYPE_LANE_CHANGE;
		default:	return METRICS_TYPE_FLOWDENSITY;
	}
}
