#ifndef _VEHICLE_H
#define _VEHICLE_H

#include "stdafx.h"
#include "IDM.h"
#include <vector>
#include <string>
#include <fstream>
#include <strstrea.h>
#include "Axle.h"

	/** A base class from which specific Vehicle types are derived*/

class Vehicle 
{

public:
	
	void init(int RoadLength, bool DriveOnRIght);
	int getLaneNoInDirection();
	void setTotalNoLanesInRoad(int nLanes);
	
	Vehicle();
	virtual ~Vehicle();
	
	// the sets
	void	setTime(double time);
	void	setChangeStatus(bool stat);
	void	setRoadPos(double r);
	void	setPos(double p);
	// desired velocity set?
	void	setAccel(double acceler);
	void	setLength(double length);
	void	setVelocity(double velocity);
	void	setLane(int l);	
	void	setDirection(bool DirPointsRight); //int d);
	void	setGVW(double weight);
	void	setAxles();
	void	setAW(int i, double w);
	void	setAxles(int axNo);
	void	setAS(int i, double s);
	
	// the gets
	CString getDataString();
	virtual double getTime() const;
	void	setRoadLength(int length);
	bool	getChangeStatus();	
	double	getRoadPos();
	double	getPos();
	double	getDesiredVel();	// why is there no set for this??
	double	getAccel();
	double	getLength();
	double	getVelocity();
	int		getLane();
	bool	getDirection();
	WORD	getID();
	double	getGVW();
	int		getNoAxles();
	double	getAW(int i);
	double	getAS(int i);

	DriverModel* getDriver();
	void	setDriver(IDM driver);	
	
	void	createSAFTVehicle(std::string data);
	void	createCASTORVehicle(std::string data);
	void	writeSAFTData(char* pTruck);
	void	writeCASTORData(char* pTruck);

	void	updateProperties(double step, double accel);
	void	update(double step, Vehicle *pV);
	void	update(double step);
	
	double	calcAccel();
	bool	changeTime(double step);
	double	calcAccel(Vehicle* pV);
	
	int		DecideNextLane(double LeftAdv, double RightAdv);
	double	DecideLaneChange(Vehicle* frontVeh, Vehicle* backVeh, bool overtake);	
	int		DecideLaneChange(Vehicle* LeftFrontVehicle, Vehicle* LeftBackVehicle,
							 Vehicle* RightFrontVehicle, Vehicle* RightBackVehicle,
							 bool LeftLaneExists,		bool RightLaneExists);


protected:
	int Round(double val) {return int(val + 0.5);};
	
	WORD	setID();
	double	LaneChangeAdvantage(Vehicle* FrontVehicle, Vehicle* BackVehicle, bool overtake);
	
	DriverModel*	m_pDriver;
	IDM				m_IDMDriver;

	bool	m_DriveOnRight;
	bool	m_DirPos;
	int		m_intDir;		// old way of identifying dirs
	int		m_Lane;
	int		m_RoadLength;
	double	m_Position;
	double	m_RoadPosition;	//used for displaying the vehicle's road/direction relative position 
	
	double	m_Velocity;
	double	m_Acceleration;
	
	int		m_Order;
	int		m_Head;
	
	int		m_Year;
	int		m_Month;	
	int		m_Day; 	
	int		m_Hour; 
	int		m_Min;
	int		m_Sec; 
	int		m_Hndt;

	double	m_GVW;
	int		m_Trns;
	int		m_NoAxles;
	double	m_Length;
	std::vector<Axle*> m_vAxles;

	WORD	m_ID;
	bool	m_bLaneChange;
	double	m_Tdelay;
	int		m_TotalNoLanesInRoad;

	int		ROAD_END_BUFFER;
	double	T_DELAY;
	double	MIN_SPACE_FOR_NEXT_VEHICLE;
	double	SAFE_BRAKING;
	double	CRANE_AVERAGE_SPACING;
	double	CRANE_MAX_SPACING;
	double	LOWLOADER_MIN_MAX_SPACING;
	int		SMALL_TRUCK_NO_AXLES;
	int		DAYS_PER_MT;
	int		MTS_PER_YR;

};
#endif
